#include <stdlib.h>
#include <math.h>
#include <streambuf>
#include <iostream>
#include <ctype.h>
#include <time.h>
#include <unistd.h>
#include "mymath.h"
#include "myassert.h"
#include "crab.h"
#include "flag_collection.h"

using namespace std;

double RUNNING_STEP		= 0.0;
double RADIUS			= 2.0;
double MUTATION_RATE		= 0.0;

inline double min_mag(double x, double y, double z) {
	x = (fabs(x) < fabs(y)) ? x : y;
	x = (fabs(x) < fabs(z)) ? x : z;
	return x;
}

inline double random_mutation(const double x) {
	// random drift, with a constant coefficient of variation
	// a uniform distribution on [ -a/2, a/2] has variance a^2/12
	//allele = allele * ( 1.0 + (uniform_random() - 0.500) * MUTATION_RATE);
	//double y = ( 1.0 + (uniform_random() - 0.500) * MUTATION_RATE/12);
	//	This drift method is strongly biased downwards, 
	//	and thus should be avoided!

	///////////////////////////////
	//  The math of calculating the equilibrium behavior was too complicated
	//  with the above mutation model.  This one is slower but simplier.
	//  Since the exponential drift term has a non-zero expectation,
	//  we have to adjust for it, by dividing by the expect value sinh(m)/m.
	//double y = exp( (2.0 * uniform_random()-1.0-MUTATION_RATE/6.0) * MUTATION_RATE);

	// Attempt to explicitly adjust for the mean
	//double mean = sinh(MUTATION_RATE)/MUTATION_RATE;
	//double y = exp( (2.0*uniform_random()-1.0) * MUTATION_RATE)/mean;

	while ( true ) {
		double y = x*exp( (2.0*uniform_random()-1.0) * MUTATION_RATE);
		if ( y > GENE1_MIN && y < GENE1_MAX ) return y;
	}
}

int main(int argc, char** argv) {
	// Initializations
	//cout.flags(ios::left);
	//cout.width(10);
	flag_collection* flags = new flag_collection(argc, argv);
	cout.precision(5);
	srandom(flags->random_seed);
	//cout << "#\tRandom seed:\t" << flags->random_seed << endl;
	if (flags->scientific) cout.flags(ios::scientific);
	RUNNING_STEP = flags->escape_velocity;
	if ( RUNNING_STEP > 0.0 ) {
		if (flags->radius > -0.5) RADIUS = flags->radius;
		else RADIUS = 2.0 * RUNNING_STEP;
	}
	MUTATION_RATE = flags->mutation_rate * log(10.0);
		// log is ln, not log10
	const int TIME_FOR_ESCAPE = flags->escape_time;
	const int NUMBER = flags->number_of_crabs;
	crab* herd[NUMBER+1];
	// The first generation starts out as a pure-breed
	for (int i=0; i<NUMBER; ++i)
		herd[i] = new crab(flags->gene1_start);
	herd[NUMBER]=NULL;

	Assert(MAX_X - MIN_X > 0, "Inverted boundaries");
	Assert(MAX_Y - MIN_Y > 0, "Inverted boundaries");
	// Main predation loop
	for (int p = -1 * flags->predation_discards; p<flags->predation_events; p++) {
		//cerr << "  Reset" << endl;
		for (int i=0; i<NUMBER; ++i) herd[i]->reset_galileon();
		//cerr << "Flee" << endl;
		for (int t = 0; t< TIME_FOR_ESCAPE; ++t) {
			// Do nil repeatedly
		}
		//cerr << "  Output before attack" << endl;
		if (p > -1 && 0 == p % flags->print_period) {
			switch (flags->output) {
				case AVE: {
					//Calculate gene1 statistics
					double E = 0; double EE = 0;
					// set gmin to the largest posible value
					// set gmax to the smallest possible value
					//double gmin = 1/GENE1_MIN;
					//double gmax = 1/GENE1_MAX;
					double gmin = GENE1_MAX;
					double gmax = GENE1_MIN;
					for (int i=0; i<NUMBER; ++i) {
						//double x = 1/herd[i]->get_gene1();
						double x = herd[i]->get_gene1();
						E += x;
						EE += x*x;
						if ( gmax < x) gmax = x;
						if ( gmin > x) gmin = x;
					}
					E = E / NUMBER;
					EE = EE / NUMBER;
					if (flags->tabs) { //output just the mean
						if ( p==0 ) cout << E;
						else cout << '\t' << E;
						cout.flush();
					} else { // output all major statistics
						cout	<< E <<'\t'
							<< sqrt(EE-E*E) << '\t'
							<< gmin << '\t'
							<< gmax
							<< endl;
					}
					break;
				} case DIST: {
					if (flags->tabs) {
						cout << 1/herd[0]->get_gene1();
						for (int i=1; i<NUMBER; i++) {
							//cout << '\t' << 1/herd[i]->get_gene1();
							cout << '\t' << herd[i]->get_gene1();
						}
						cout << endl;
					} else {
						for (int i=0; i<NUMBER; i++) {
							//cout << 1/herd[i]->get_gene1() << endl;
							cout << herd[i]->get_gene1() << endl;
						} cout << endl;
					}
					break;
				} case POS: {
					for (int i=0; i<NUMBER; ++i) {
						if (flags->tabs) {
							Assert(false, "no matlab format yet");
						} else {
							cout	<< herd[i]->get_x() << '\t'
									<< herd[i]->get_y() << endl;
						}
					} cout << endl << endl;
					break;
				} default:
					cerr << "Unimplemented output style" << endl;
			}
		}
		//cerr << "  Death" << endl;
		// Predator appears..
		double predator_x = uniform_random() * SIZE_X + MIN_X;
		double predator_y = uniform_random() * SIZE_Y + MIN_Y;
		// Finds the nearest crab..
		int closest_crab = NUMBER;
		double distance_to_closest_crab = SIZE_X + SIZE_Y;
		for (int i=0; i < NUMBER; ++i ) {
			double tmpx = predator_x - herd[i]->get_x();
			double tmpy = predator_y - herd[i]->get_y();
			tmpx = min_mag( tmpx - SIZE_X, tmpx, tmpx + SIZE_X);
			tmpy = min_mag( tmpy - SIZE_Y, tmpy, tmpy + SIZE_Y);
			double tmpd = hypot(tmpx,tmpy);
			if (tmpd < distance_to_closest_crab) {
				closest_crab = i;
				distance_to_closest_crab = tmpd;
			}
		} Assert( closest_crab < NUMBER, "No crab eatten");
		// And Eats!
		//delete *( herd[closest_crab] );

		//pick parent
		int random_parent = closest_crab;
		while ( random_parent == closest_crab ) {
			random_parent = int_random(NUMBER);
			Assert( random_parent < NUMBER && random_parent > -1,
				"Bad random integer");
		}
		// mutate
		double allele = herd[random_parent]->get_gene1();
		allele = random_mutation(allele);
		// birth
		herd[closest_crab]->set_gene1(allele);
	}
	cout << endl;
	exit(0);
}
